/* 
 *  Module Name: macapi.h
 *  Author: Joe Bonaker
 *
 *  Copyright  1997-1999 Conexant Systems, Inc.
 *  All rights reserved.
 *
 *  This software may be used and distributed according to the terms
 *  of the GNU Public License, incorporated herein
 */

#if !defined(MACAPI_H_)
#define MACAPI_H
#endif

#include <linux/netdevice.h>
#include <linux/etherdevice.h>
//#include "linuxinc.h"

/* Keep the ring sizes a power of two for efficiency.
   Making the Tx ring too large decreases the effectiveness of channel
   bonding and packet priority.
   There are no ill effects from too-large receive rings. */
/* #define TX_RING_SIZE	8 */
#define TX_RING_SIZE	8
#define RX_RING_SIZE	32

/* A table describing the chip types. */
static struct rss_chip_table {
  int device_id;
  int rev_id;
  char *chip_name;
  int flags;
  void (*media_timer)(unsigned long data);
} rss_tbl[] = {
//  { RSS610_DEVID, RSS_REV1, "11610", 0, rss_timer },	/* 11610 */
//  { RSS611_DEVID, RSS_REV3, "11611", 0, rss_timer },	/* 11611 */
//  { RSS611_DEVID, RSS_REV4, "11611", 0, rss_timer },	/* 11611 */
//  { RSS617_DEVID, RSS_REV5, "11617", 0, rss_timer },	/* 11617 */
  { RSS623_DEVID, RSS_REV8, "11623", 0, rss_timer },	/* 11623 */
  {0, 0, 0, 0},
};

/* Offsets to the Command and Status Registers, "CSRs".  All accesses
   must be longword instructions and quadword aligned. */
/*
enum rss_offsets {
	RSS_PAR=0, RSS_TDR=0x08, RSS_RDR=0x10, RSS_RDB=0x18, RSS_TDB=0x20, RSS_SR1=0x28,
	RSS_NAR=0x30, RSS_IER1=0x38, RSS_LPC=0x40, RSS_SPR=0x48, RSS_TMR=0x58,
	RSS_GPIO=0x60, RSS_WTMR=0x78 };
*/

/* The RWNIC Rx and Tx buffer descriptors. */
struct rss_rx_desc {
	s32 status;
	s32 length;
	u32 buffer1, buffer2;
};

struct rss_tx_desc {
	s32 status;
	s32 length;
	u32 buffer1, buffer2;				/* We use only buffer 1.  */
};

struct rss_private {
	char devname[8];			/* Used only for kernel debugging. */
	const char *product_name;
	struct net_device *next_module;
	struct rss_rx_desc rx_ring[RX_RING_SIZE];
	struct rss_tx_desc tx_ring[TX_RING_SIZE];
	/* The saved address of a sent-in-place packet/buffer, for skfree(). */
	struct sk_buff* tx_skbuff[TX_RING_SIZE];
	/* The addresses of receive-in-place skbuffs. */
	struct sk_buff* rx_skbuff[RX_RING_SIZE];
	char *rx_buffs;				/* Address of temporary Rx buffers. */
	int setup_frame[48];		/* Pseudo-Tx frame to init address table. */
	int chip_id;
	int revision;
	struct net_device_stats stats;
	struct timer_list timer;	/* Media selection timer. */
	unsigned int cur_rx, cur_tx;		/* The next free ring entry */
	unsigned int dirty_rx, dirty_tx;	/* The ring entries to be free()ed. */
	unsigned int tx_full:1;				/* The Tx queue is full. */
	unsigned int full_duplex:1;			/* Full-duplex operation requested. */
	unsigned int default_port:4;		/* Last dev->if_port value. */
	unsigned int rss_nar;					/* Current RSS_NAR control settings. */
	unsigned char eeprom[512];			/* Serial EEPROM contents. */
    PHY           phy;
	int			tx_timeout;				/*For Tx hang */
	int			CableModem;				/* 1: for cable modem */
};

