/*
 *  Module Name: rss.h
 *  Author: Joe Bonaker
 * 
 *  Copyright  1998 Conexant Systems, Inc.
 *  All rights reserved.
 *
 *  This software may be used and distributed according to the terms
 *  of the GNU Public License, incorporated herein
 */

extern unsigned int rss_units;

typedef unsigned char		UCHAR;
typedef unsigned int		UINT;
typedef unsigned long int	ULONG;

#define RSS_TIME_OUT		200

//#define PCI_VENDOR_ID_RSS	0x127A	/* PCI Vendor ID */
#define PCI_VENDOR_ID_RSS	0x14F1	/* PCI Vendor ID */

#define RSS610_DEVID 0x1800	   /* RSS610 PCI Device ID */
#define RSS611_DEVID 0x1801	   /* RSS611 PCI Device ID */
#define RSS617_DEVID 0x1801	   /* RSS617 PCI Device ID */
#define RSS623_DEVID 0x1803	   /* RSS623 PCI Device ID */
#define RSS_BASE_ADDR 0x10         /* Offset in PCI space of base address */
#define RSS_ILINE 0x3c             /* Offset in PCI space of IRQ */
#define RSS_REV 0x08		   /* Offset in PCI space of Revision */
#define RSS_BASE_ADDRMASK 0xfffffffe

enum rss_rev {
	RSS_REV0,
	RSS_REV1,
	RSS_REV2,
	RSS_REV3,
	RSS_REV4,
	RSS_REV5,
	RSS_REV6,
	RSS_REV7,
	RSS_REV8,
	};

#define RSS_ETHER_MTU	1514        /* Ethernet MTU */
#define RSS_MINPACK 50             /* Minimum packet size */
#define RSS_NMCADDR 14             /* Number of multicast addrs */

#define SROM_SIZE 512               /* Size of the SROM */

#define SROM_VER3_ETHER 0x86        /* SROM 610, 611 ver3, ethernet addr offs */
#define SROM_VER4_ETHER 0x186	    /* SROM 611 ver 4, ethernet addr offset */
#define SROM_VER5_ETHER 0x196
#define SROM_VER8_ETHER 0x19A

#define SROM_READ_OP 0x4000         /* SROM read operation */
#define SROM_SWO 0x2000		    /* SROM write operatione */
#define SROM_SEL 0x800              /* CSR9 select SROM */
#define CHIP_SEL 0x01               /* Chip select */
#define CLK_SEL 0x02                /* Clock select */
#define EE_R0_LOW 0x4801
#define EE_R0_HI 0x4803
#define EE_R1_LOW 0x4805
#define EE_R1_HI 0x4807
#define EE_SET_LOW 0x4801
#define DATA_IN 0x04                /* CSR9 data in */
#define DATA_OUT 0x08               /* CSR9 data out */

#define RSS_MAX_XMITS 300                  /* Maximum number of buffer/descriptors */
#define RSS_MAX_RECVS 300


#define TDES_EOR 0x02000000                 /* Descriptor end-of-ring bits */
#define RDES_EOR 0x02000000

#define RDES_OWN 0x80000000    /* Descriptor ownership bit.  On means 1161x owns */
#define TDES_OWN 0x80000000

#define TDES_ES 0x00008000     /* Error summaries and frame errors */
#define RDES_ES 0x00008000
#define RDES_FE 0x00004000

#define TDES_FIRST 0x20000000  /* Descriptor first frame segment and last */
#define TDES_LAST 0x40000000

#define RSS_SETUPBUF_SIZE 192         /* Bits for setup packet */
#define RSS_SETUP_PACKET  0x08000000
#define RSS_PERFECT_FILTER 0

#define INTR_NORMAL_SUMM 0x10000       /* Interrupt enable bits */
#define INTR_ABNORMAL_SUMM 0x8000
#define INTR_GTE 0x0800
#define INTR_RX 0x40
#define INTR_TX 0x05
#define RX_STATE_4 0x80000
#define TX_HANG 0x200000
#define TX_DMAING 0x300000

#define RECV_LEN_SHIFT 16              /* For getting frame length */
#define RECV_LEN_MASK 0x3fff0000

/* Xmit error bits */

#define XMIT_EXCOLL 0x100
#define XMIT_CARRIER 0x400

/* Recv error bits */

#define RECV_CRC 0x2


#define RSS_POLL_DEMAND 0x01         /* Effect a poll (xmit or receive) */

#define START_XMIT 0x2000             /* Start the transmitter (omr) */
#define START_RECV 0x2                /* Start the receiver (omr) */

#define RSS_SW_RESET 1               /* Soft reset (bmr) */
#define RSS_BURST 0x5000             /* Burst size */

/* Operating mode register bits */

#define NAR_PS 0x40000                /* Port select */
#define NAR_PCS 0x800000              /* PCS Function */
#define NAR_PROM 0x40                 /* Promiscuous mode */
#define NAR_SCR 0x01000000            /* Scrambler mode */
#define NAR_HBD 0x00080000            /* Heatbeat disable */
#define NAR_FD 0x200                  /* Full duplex */
#define NAR_TTM_100 0x0               /* Transmit threshold set for 100mbit */
#define NAR_TTM_10 0x400000           /* Transmit threshold set for 10mbit */
#define NAR_MB1 0x2000000             /* Must be one */
#define NAR_MB0 0x1c000000            /* Must be zero */
#define NAR_TSF 0x200000	      /* Transmit Store and Forward */


/* (RSS610,RSS611) Register offsets (from base address) */

#define RSS_SPR 0x48                 /* ROM Register */
#define RSS_PAR 0x00                 /* Bus Mode */
#define RSS_NAR 0x30                 /* Operating mode */
#define RSS_LPC 0x40		     /* Lost Packet Counter */
#define RSS_TDB 0x20                 /* Transmit list */
#define RSS_SR1 0x28                 /* Interrupt status */
#define RSS_IER1 0x38                /* Interrupt mask */
#define RSS_RDB 0x18                 /* Receive list */
#define RSS_TDR 0x08                 /* Transmit poll */
#define RSS_RDR 0x10                 /* Receive poll */
#define RSS_GPIO 0x60                /* General purpose port */
