/* rss61x.c: A Conexant LANfinity Fast Ethernet driver for linux. */
/*
	This driver is based on the codes developed by Donald Becker.
	This driver is for the Conexant PCI ethernet adapter.

*/

/* tulip.c: A DEC 21040-family ethernet driver for linux. */
/*
   NOTICE: THIS IS THE ALPHA TEST VERSION!
	Written 1994-1997 by Donald Becker.

	This software may be used and distributed according to the terms
	of the GNU Public License, incorporated herein by reference.

	This driver is for the SMC EtherPower PCI ethernet adapter.
	It should work with most other DEC 21*40-based ethercards.

	The author may be reached as becker@CESDIS.gsfc.nasa.gov, or C/O
	Center of Excellence in Space Data and Information Sciences
	   Code 930.5, Goddard Space Flight Center, Greenbelt MD 20771

	Support and updates available at
	http://cesdis.gsfc.nasa.gov/linux/drivers/tulip.html
*/

/* static const char *version = "tulip.c:v0.73 4/16/97 becker@cesdis.gsfc.nasa.gov\n"; */

/*
 * Modification History
 *
 * 07/01/1997 - Initial port from DEC Tulip to RSS11606 MAC and EEPROM
 * 04/08/1998 - Port to new RSS11617 MAC device and PHY driver
 * 04/03/2000 - Port to RS7112 MAC device and Red Hat 6.1 kernel
 * 04/14/2000 - Alpha PCI-CM: removed PHY driver support for PCI Cable Modem use
 * 05/16/2000 - Kernel 2.2.14.5-0 version build
 * 05/31/2000 - Beta PCI-CM release
 * 03/06/2002 - Moved some code into macapi.h 
 */


static const char *version = "cn7112:v1.00 04/12/00\n";

// A few user-configurable values.

#define SROM_SIZE 512

// Set the copy breakpoint for the copy-only-tiny-buffer Rx structure.
#define SKBUFF_RX_COPYBREAK 200

// The following example shows how to always use the 10base2 port.
#ifdef notdef
#define RSS_DEFAULT_MEDIA 1		// 1 == 10base2
#define RSS_NO_MEDIA_SWITCH		// Don't switch from this port
#endif


// Operational parameters that usually are not changed.
// Time in jiffies before concluding the transmitter is hung.
#define TX_TIMEOUT  ((2000*HZ)/1000)

#include <linux/config.h>
#ifdef MODULE
#ifdef MODVERSIONS
#include <linux/modsetver.h>
#include <linux/modversions.h>
#endif
#include <linux/module.h>
#include <linux/version.h>
#else
#define MOD_INC_USE_COUNT
#define MOD_DEC_USE_COUNT
#endif

#ifdef VER_CTL
#include <linux/autoconf.h>
#if defined(CONFIG_MODVERSIONS) && !defined(MODVERSIONS)
#define MODVERSIONS			// force it on
#endif
#endif

#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/string.h>
#include <linux/timer.h>
#include <linux/ptrace.h>
#include <linux/errno.h>
#include <linux/ioport.h>
#include <linux/malloc.h>
#include <linux/interrupt.h>
#include <linux/pci.h>
#include <asm/processor.h>		// Processor type for cache alignment
#include <asm/bitops.h>
#include <asm/io.h>
#include <asm/dma.h>
#include <asm/unaligned.h>

static void rss_timer(unsigned long data);

#include <linux/skbuff.h>
#include "phy.h"
#include "rss.h"
#include "macapi.h"

// Kernel compatibility defines, common to David Hind's PCMCIA package.
//   This is only in the support-all-kernels source code.
#include <linux/version.h>		// Evil, but neccessary

#if defined (LINUX_VERSION_CODE) && LINUX_VERSION_CODE < 0x10300
#define RUN_AT(x) (x)			// What to put in timer->expires.
#define DEV_ALLOC_SKB(len) alloc_skb(len, GFP_ATOMIC)
#define virt_to_bus(addr)  ((unsigned long)addr)
#define bus_to_virt(addr) ((void*)addr)

#else  // 1.3.0 and later
#define RUN_AT(x) (jiffies + (x))
#define DEV_ALLOC_SKB(len) dev_alloc_skb(len + 2)
#endif

#if LINUX_VERSION_CODE < 0x20159
#define dev_free_skb(skb) do {dev_kfree_skb(skb, FREE_WRITE); if (rss_debug > 5) printk("dev_kfree_skb called\n"); } while(0)
#else
#define dev_free_skb(skb)  do {dev_kfree_skb(skb); if (rss_debug > 5) printk("dev_kfree_skb called\n"); } while(0)
#endif

#if LINUX_VERSION_CODE >= 0x020400
#define device net_device
#endif

#if (LINUX_VERSION_CODE >= 0x20100)
char kernel_version[] = UTS_RELEASE;
#endif

#if LINUX_VERSION_CODE <= 0x20300
#define	net_device_stats enet_statistics
#define	net_device device
#else
#define NETSTATS_VER2
#endif

#if LINUX_VERSION_CODE < 0x20155
#include <linux/bios32.h>
#define PCI_SUPPORT_VER1
#else
#define PCI_SUPPORT_VER2
#endif

#if (LINUX_VERSION_CODE >= 0x10344)
#define NEW_MULTICAST
#include <linux/delay.h>
#endif

#ifdef SA_SHIRQ
#define FREE_IRQ(irqnum, dev) free_irq(irqnum, dev)
#define REQUEST_IRQ(i,h,f,n, instance) request_irq(i,h,f,n, instance)
#define IRQ(irq, dev_id, pt_regs) (irq, dev_id, pt_regs)
#else
#define FREE_IRQ(irqnum, dev) free_irq(irqnum)
#define REQUEST_IRQ(i,h,f,n, instance) request_irq(i,h,f,n)
#define IRQ(irq, dev_id, pt_regs) (irq, pt_regs)
#endif

// This my implementation of shared IRQs, now only used for 1.2.13.
#ifdef HAVE_SHARED_IRQ
#define USE_SHARED_IRQ
#include <linux/shared_irq.h>
#endif

#define RSS_TOTAL_SIZE 0xF0

#ifdef HAVE_DEVLIST
struct netdev_entry rss_drv =
{"RSS", rss_pci_probe, RSS_TOTAL_SIZE, NULL};
#endif

#ifdef RSS_DEBUG
int rss_debug = RSS_DEBUG;
#else
//int rss_debug = 1;
int rss_debug = 0;		//for release
#endif

#define DESC_OWNED_BY_HOST	0				// ready to TX
#define DESC_OWNED_BY_MAC	0x80000000	// ready to RX or IN TX


/*
				Theory of Operation

I. Board Compatibility

This device driver is designed for the Rockwell LANfinity single-chip ethernet
controllers for PCI.  


II. Board-specific settings

PCI bus devices are configured by the system at boot time, so no jumpers
need to be set on the board.  The system BIOS preferably should assign the
PCI INTA signal to an otherwise unused system IRQ line.
Note: Kernel versions earlier than 1.3.73 do not support shared PCI
interrupt lines.

III. Driver operation

IIIa. Ring buffers

The Rockwell NIC can use either ring buffers or lists of Tx and Rx descriptors.
This driver uses statically allocated rings of Rx and Tx descriptors, set at
compile time by RX/TX_RING_SIZE.  This version of the driver allocates skbuffs
for the Rx ring buffers at open() time and passes the skb->data field to the
NIC as receive data buffers.  When an incoming frame is less than
SKBUFF_RX_COPYBREAK bytes long, a fresh skbuff is allocated and the frame is
copied to the new skbuff.  When the incoming frame is larger, the skbuff is
passed directly up the protocol stack and replaced by a newly allocated
skbuff.

The SKBUFF_RX_COPYBREAK value is chosen to trade-off the memory wasted by
using a full-sized skbuff for small frames vs. the copying costs of larger
frames.  For small frames the copying cost is negligible (esp. considering
that we are pre-loading the cache with immediately useful header
information).  For large frames the copying cost is non-trivial, and the
larger copy might flush the cache of useful data.  A subtle aspect of this
choice is that the NIC only receives into longword aligned buffers, thus
the IP header at offset 14 isn't longword aligned for further processing.
Copied frames are put into the new skbuff at an offset of "+2", thus copying
has the beneficial effect of aligning the IP header and preloading the
cache.

IIIC. Synchronization
The driver runs as two independent, single-threaded flows of control.  One
is the send-packet routine, which enforces single-threaded use by the
dev->tbusy flag.  The other thread is the interrupt handler, which is single
threaded by the hardware and other software.

The send packet thread has partial control over the Tx ring and 'dev->tbusy'
flag.  It sets the tbusy flag whenever it's queuing a Tx packet. If the next
queue slot is empty, it clears the tbusy flag when finished otherwise it sets
the 'dv->tx_full' flag.

The interrupt handler has exclusive control over the Rx ring and records stats
from the Tx ring.  (The Tx-done interrupt can't be selectively turned off, so
we can't avoid the interrupt overhead by having the Tx routine reap the Tx
stats.)	 After reaping the stats, it marks the queue entry as empty by setting
the 'base' to zero.	 Iff the 'dv->tx_full' flag is set, it clears both the
tx_full and tbusy flags.

*/
// A few values that may be tweaked.
#define PKT_BUF_SZ		1536			// Size of each temporary Rx buffer.

// Delay between EEPROM clock transitions.
// The 1.2 code is a "nasty" timing loop, but PC compatible machines are
// *supposed* to delay an ISA-compatible period for the SLOW_DOWN_IO macro.
#ifdef _LINUX_DELAY_H
#define rss_delay(nanosec)	udelay((nanosec + 999)/1000)
#else
#define rss_delay(nanosec)	do { int _i = 3; while (--_i > 0) { __SLOW_DOWN_IO; }} while (0)
#endif

static struct device *rss_probe1(struct device *dev, int ioaddr, int irq, int chip_id, int options);
static int rss_phy_check(struct device *dev);
static void rss_read_srom(struct device *dev, unsigned char *srom);
static int rss_hw_init(struct device *dev);
static void rss_setup_mem(struct device *dev);
static void rss_reset(struct device *dev);
static int rss_open(struct device *dev);
static void rss_timer(unsigned long data);
static void rss_tx_timeout(struct device *dev);
static void rss_init_ring(struct device *dev);
static int rss_start_xmit(struct sk_buff *skb, struct device *dev);
static int rss_rx(struct device *dev);
static void rss_interrupt IRQ(int irq, void *dev_instance, struct pt_regs *regs);
static int rss_close(struct device *dev);
static struct net_device_stats *rss_get_stats(struct device *dev);
#ifdef NEW_MULTICAST
static void set_multicast_list(struct device *dev);
#else
static void set_multicast_list(struct device *dev, int num_addrs, void *addrs);
#endif

#ifdef MODULE
/* Used to pass the full-duplex flag, etc. */
static int options[] = {-1, -1, -1, -1, -1, -1, -1, -1};
#endif


#ifdef MODULE
// A list of all installed Rockwell NIC devices, for removing the driver module.
static struct device *root_rss_dev = NULL;
#endif

int rss_probe( struct device *dev )
{
	int cards_found = 0;
	static int pci_index = 0;	// Static, for multiple probe calls.
#if LINUX_VERSION_CODE >= 0x020400
  int enable_result;
#endif

	// Ideally we would detect all network cards in slot order.  That would
	// be best done a central PCI probe dispatch, which wouldn't work
	// well with the current structure.  So instead we detect just the
	// RWNIC cards in slot order.
	
	if ( pcibios_present() )
	{
		unsigned char pci_bus, pci_device_fn;

		for ( ; pci_index < 0xff; pci_index++ )
		{
			unsigned char pci_irq_line, pci_latency;
			unsigned short pci_command, vendor, device;
 			unsigned short rev_id;
			unsigned int pci_ioaddr, chip_idx = 0;

#ifdef REVERSE_PROBE_ORDER
			if ( pcibios_find_class(PCI_CLASS_NETWORK_ETHERNET << 8,
									0xfe - pci_index,
									&pci_bus, &pci_device_fn) != PCIBIOS_SUCCESSFUL )
			{
				// not a ethernet device at this 0xfe - pci_index value
				continue;
			}
#else
			if ( pcibios_find_class(PCI_CLASS_NETWORK_ETHERNET << 8,
									pci_index,
									&pci_bus, &pci_device_fn ) != PCIBIOS_SUCCESSFUL )
			{
				// not a ethernet device at this pci_index value
				break;
			}
#endif
			// found an ethernet device, is it ours?
			
			// read PCI config space
			pcibios_read_config_word(pci_bus, pci_device_fn, PCI_VENDOR_ID, &vendor);
			pcibios_read_config_word(pci_bus, pci_device_fn, PCI_DEVICE_ID, &device);
			pcibios_read_config_word(pci_bus, pci_device_fn, PCI_COMMAND_SPECIAL, &rev_id);
			
			if( vendor != PCI_VENDOR_ID_RSS )
			{
				// not our vendor id number
				continue;
			}

		{
			#if defined(PCI_SUPPORT_VER2)
				// get pci_dev structure
				struct pci_dev *pdev = pci_find_slot(pci_bus, pci_device_fn);
				
				// get the base memory/IO address of our device
				#if LINUX_VERSION_CODE >= 0x020400
                                        /*under 2.4, you have to explicitly enable PCI devices*/
                                        enable_result = pci_enable_device(pdev);
                                        if (enable_result) {
                                           printk("Cannot enable PCI cable modem\n");
                                           return enable_result;
                                        }
					pci_ioaddr = pci_resource_start(pdev, 0) ;
                                        pci_ioaddr &= ~3;
				#else
					pci_ioaddr = pdev->base_address[0] & ~3;
				#endif
				
				// get the IRQ number of our device
				pci_irq_line = pdev->irq;
			#else
				// get the IRQ number of our device
				pcibios_read_config_byte(pci_bus, pci_device_fn, PCI_INTERRUPT_LINE, &pci_irq_line);
				// get the base memory/IO address of our device
				pcibios_read_config_dword(pci_bus, pci_device_fn,PCI_BASE_ADDRESS_0, &pci_ioaddr);
				// Remove I/O space marker in bit 0.
				pci_ioaddr &= ~3;
			#endif
		}

			for ( chip_idx = 0; rss_tbl[chip_idx].chip_name; chip_idx++ )
			{
				// is this our device id ?
				if ( device == rss_tbl[chip_idx].device_id )
				{
					// is this the right chip rev ?
					if ( rev_id == rss_tbl[chip_idx].rev_id )
					{
						// yes
						break;
					}
				}
			}
				
			if ( rss_tbl[chip_idx].chip_name == 0 )
			{
				printk("Unknown RSS PCI ethernet chip type %4.4x detected:"
					   " not configured.\n", device);
				continue;
			}
			
			if ( rss_debug > 2 )
			{
				printk("Found RSS PCI RWNIC at I/O %#x, IRQ %d.\n",
					   pci_ioaddr, pci_irq_line);
			}

			// is the I/O port address range available for allocation?
			if ( check_region(pci_ioaddr, RSS_TOTAL_SIZE) )
			{
				continue;
			}

#ifdef MODULE
			dev = rss_probe1(dev, pci_ioaddr, pci_irq_line, chip_idx, options[cards_found]);
#else
			dev = rss_probe1(dev, pci_ioaddr, pci_irq_line, chip_idx, dev ? dev->mem_start : 0);
#endif
			if ( dev )
			{
			  // Get and check the bus-master and latency values.
			  pcibios_read_config_word(pci_bus, pci_device_fn, PCI_COMMAND, &pci_command);
			  if ( !(pci_command & PCI_COMMAND_MASTER) )
			  {
				printk("  PCI Master Bit has not been set! Setting...\n");
				pci_command |= PCI_COMMAND_MASTER;
				pcibios_write_config_word(pci_bus, pci_device_fn, PCI_COMMAND, pci_command);
			  }
			
			  // Get and check the pci bus latency
			  pcibios_read_config_byte(pci_bus, pci_device_fn, PCI_LATENCY_TIMER, &pci_latency);
			  if ( pci_latency < 10 )
			  {
				printk("  PCI latency timer (CFLT) is unreasonably low at %d."
					   "  Setting to 255 clocks.\n", pci_latency);
				pcibios_write_config_byte(pci_bus, pci_device_fn, PCI_LATENCY_TIMER, 255);
			  }
			  else if ( rss_debug > 1 )
			  {
				printk("  PCI latency timer (CFLT) is %#x.\n", pci_latency);
			  }
			
			  dev = 0;
			  cards_found++;
			}
		}
	}

#if defined (MODULE)
	return cards_found;
#else
	return 0;
#endif
}


static struct device *rss_probe1( struct device *dev, int ioaddr, int irq,
								  int chip_id, int options )
{
	struct rss_private *dv;
	// See note below on the multiport cards.
	static unsigned char last_phys_addr[6] = {0x00, 'L', 'i', 'n', 'u', 'x'};
	static int last_irq = 0;
	int i;
	unsigned short sum;
	int sa_offset;

	dev = init_etherdev(dev, sizeof(struct rss_private));

	printk("%s: RW %s Rwnic at %#3x,",
		   dev->name, rss_tbl[chip_id].chip_name, ioaddr);

	// Stop the chip's Tx and Rx processes.
	outl(inl(ioaddr + RSS_NAR) & ~0x2002, ioaddr + RSS_NAR);
	// Clear the missed-packet counter.
	inl(ioaddr + RSS_LPC);
	
	/* We do a request_region() only to register /proc/ioports info. */
	request_region(ioaddr, RSS_TOTAL_SIZE, rss_tbl[chip_id].chip_name);

	dev->base_addr = ioaddr;
	dev->irq = irq;

	// Make certain the data structures are quadword aligned.
	dv = (void *)(((long)kmalloc(sizeof(*dv), GFP_KERNEL | GFP_DMA) + 7) & ~7);
	memset(dv, 0, sizeof(*dv));
	dev->priv = dv;

	// The station address ROM is read byte serially.  The register must
	// be polled, waiting for the value to be read bit serially from the
	// EEPROM.
	
	sum = 0;

	rss_read_srom(dev,dv->eeprom);

	switch ( rss_tbl[chip_id].rev_id )
	{
	  case RSS_REV3:
		sa_offset = SROM_VER3_ETHER;
		break;
	  case RSS_REV4:
		sa_offset = SROM_VER4_ETHER;
		break;
	  case RSS_REV5:
		sa_offset = SROM_VER5_ETHER;
		break;
	  case RSS_REV8:
		sa_offset = SROM_VER8_ETHER;
		break;
	  default:
		sa_offset = SROM_VER3_ETHER;
		break;

	}

	for ( i = 0; i < 6; i ++ )
	{
		dev->dev_addr[i] = dv->eeprom[i + sa_offset];
		sum += dv->eeprom[i + sa_offset];
	}
	
	for ( i = 0; i < 6; i++ )
	{
		printk(" %2.2x", last_phys_addr[i] = dev->dev_addr[i]);
	}
	printk(", IRQ %d\n", irq);
	last_irq = irq;

#ifdef MODULE
	dv->next_module = root_rss_dev;
	root_rss_dev = dev;
#endif
	dv->chip_id = chip_id;

#ifdef RSS_DEFAULT_MEDIA
	dv->default_port = RSS_DEFAULT_MEDIA;
#endif

	// The lower four bits are the media type.
	if( options > 0 )
	{
		dv->full_duplex = (options & 16) ? 1 : 0;
		dv->default_port = options & 15;
	}

	// The RWNIC-specific entries in the device structure.
	dev->open = &rss_open;
	dev->hard_start_xmit = &rss_start_xmit;
	dev->stop = &rss_close;
	dev->get_stats = &rss_get_stats;
#ifdef HAVE_MULTICAST
	dev->set_multicast_list = &set_multicast_list;
#endif

	rss_hw_init(dev);

    // During Driver initialization time, wait until Link is up
    //for ( i=0; i < 5500; i++ )
    //{
  	//	phy_data = rss_phy_check(dev);
   	//	if (phy_data & PHY_LINK_UP)
   	//    {
    //		  break;
	//    }
   	//	udelay(1000);
   	//}
      	
   	//if ( !(phy_data & PHY_LINK_UP) )
   	//{
  	//	printk ("rss: cable disconnect error.\n");
    //}

	return dev;
}


/************************************************
 * Read the entire SROM data			* 
 ************************************************/
void rss_read_srom( struct device *dev, unsigned char *srom )
{
  int i,j;
  UINT rom_addr;
  UINT bit;
  unsigned long dout;
  unsigned short word;

  rom_addr = 0;

  for ( i = 0; i < SROM_SIZE; i += 2 )
  {
    outl(SROM_READ_OP | SROM_SEL,dev->base_addr + RSS_SPR);
    rss_delay(1000);
    outl(SROM_READ_OP | SROM_SEL | CHIP_SEL,dev->base_addr + RSS_SPR);
    rss_delay(1000);
    outl(SROM_READ_OP | SROM_SEL | CHIP_SEL | CLK_SEL,dev->base_addr + RSS_SPR);
    rss_delay(1000);
    outl(SROM_READ_OP | SROM_SEL | CHIP_SEL,dev->base_addr + RSS_SPR);
    rss_delay(1000);
    outl(SROM_READ_OP | SROM_SEL | CHIP_SEL | DATA_IN,dev->base_addr + RSS_SPR);
    rss_delay(1000);
    outl(SROM_READ_OP | SROM_SEL | CHIP_SEL | DATA_IN | CLK_SEL,dev->base_addr + RSS_SPR);
    rss_delay(1000);
    outl(SROM_READ_OP | SROM_SEL | CHIP_SEL | DATA_IN,dev->base_addr + RSS_SPR);
    rss_delay(1000);
    outl(SROM_READ_OP | SROM_SEL | CHIP_SEL | DATA_IN | CLK_SEL,dev->base_addr + RSS_SPR);
    rss_delay(1000);
    outl(SROM_READ_OP | SROM_SEL | CHIP_SEL | DATA_IN,dev->base_addr + RSS_SPR);
    rss_delay(1000);
    outl(SROM_READ_OP | SROM_SEL | CHIP_SEL,dev->base_addr + RSS_SPR);
    rss_delay(1000);
    outl(SROM_READ_OP | SROM_SEL | CHIP_SEL | CLK_SEL,dev->base_addr + RSS_SPR);
    rss_delay(1000);
    outl(SROM_READ_OP | SROM_SEL | CHIP_SEL,dev->base_addr + RSS_SPR);
    rss_delay(1000);

    for ( j = 0x80; j >= 1; j >>= 1 )
    {
      bit = (rom_addr & j) ? DATA_IN : 0;
      outl(SROM_READ_OP | SROM_SEL | CHIP_SEL | bit,dev->base_addr + RSS_SPR);
      rss_delay(1000);
      outl(SROM_READ_OP | SROM_SEL | CHIP_SEL | bit | CLK_SEL,dev->base_addr + RSS_SPR);
      rss_delay(1000);
      outl(SROM_READ_OP | SROM_SEL | CHIP_SEL | bit,dev->base_addr + RSS_SPR);
    }
    rss_delay(1000);

    word = 0;
    for ( j = 0x8000; j >= 1; j >>= 1 )
    {
      outl(SROM_READ_OP | SROM_SEL | CHIP_SEL | CLK_SEL,dev->base_addr + RSS_SPR);
      rss_delay(1000);
      dout = inl(dev->base_addr + RSS_SPR);
      rss_delay(1000);
      if (dout & DATA_OUT)
      {
		word |= j;
	  }
      outl(SROM_READ_OP | SROM_SEL | CHIP_SEL,dev->base_addr + RSS_SPR);
      rss_delay(1000);
    }

    srom[i] = (word & 0x00ff);
    srom[i+1] = (word >> 8);
    rom_addr++;
    outl(SROM_READ_OP | SROM_SEL,dev->base_addr + RSS_SPR);
    rss_delay(1000);
  }
}

//Constants for the OPERATION_MODE register

#define RSS1161x_RCV_HASH_FILTER_MODE    ((ULONG)(0x00000001))
#define RSS1161x_RCV_START               ((ULONG)(0x00000002))
#define RSS1161x_PASS_BAD_FRAMES         ((ULONG)(0x00000008))
#define RSS1161x_INVERSE_FILTERING       ((ULONG)(0x00000010))
#define RSS1161x_STOP_BACKOFF_COUNTER    ((ULONG)(0x00000020))
#define RSS1161x_PROMISCUOUS_MODE        ((ULONG)(0x00000040))

#define RSS1161x_PASS_ALL_MULTICAST      ((ULONG)(0x00000080))
#define RSS1161x_FLAKY_OSC_DISABLE       ((ULONG)(0x00000100))
#define RSS1161x_FULL_DUPLEX_MODE        ((ULONG)(0x00000200))
#define RSS1161x_OPERATING_MODE          ((ULONG)(0x00000C00))
#define RSS1161x_FORCE_COLLISION_MODE    ((ULONG)(0x00001000))
#define RSS1161x_TXM_START               ((ULONG)(0x00002000))
#define RSS1161x_TXM_THRESHOLD           ((ULONG)(0x0000C000))
#define RSS1161x_BACK_PRESSURE           ((ULONG)(0x00010000))

#define RSS1161x_CAPTURE_EFFECT          ((ULONG)(0x00020000))
#define RSS1161x_INTERNAL_HLAN_SELECT    ((ULONG)(0x00020000))
#define RSS1161x_PORT_SELECT             ((ULONG)(0x00040000))
#define RSS1161x_HEARTBEAT_DISABLE       ((ULONG)(0x00080000))
#define RSS1161x_IMMEDIATE_TRANSMISSION  ((ULONG)(0x00100000))
#define RSS1161x_STORE_AND_FORWARD       ((ULONG)(0x00200000))
#define RSS1161x_TXM_THRESHOLD_MODE      ((ULONG)(0x00400000))
#define RSS1161x_NETWORK_SPEED_10		 ((ULONG)(0x00400000))
#define RSS1161x_CS_MODE                 ((ULONG)(0x00800000))
#define RSS1161x_SCRAMBLER               ((ULONG)(0x01000000))
#define RSS1161x_LINK_HYSTERESIS         ((ULONG)(0x02000000))

//RSS1161x:

#define RSS1161x_TXM_THRESHOLD_72_128    ((ULONG)(0x00000000))
#define RSS1161x_TXM_THRESHOLD_96_256    ((ULONG)(0x00004000))
#define RSS1161x_TXM_THRESHOLD_128_512   ((ULONG)(0x00008000))
#define RSS1161x_TXM_THRESHOLD_160_1024  ((ULONG)(0x0000C000))

#define RSS1161x_DEFAULT_TXM_THRESHOLD   RSS1161x_TXM_THRESHOLD_160_1024

#define PHY_HWOPTS_NIC_RSS11623         \
    (PHY_HWOPTS_ALT_1HZ_HPNA_LINK | PHY_HWOPTS_HWRESET_GP0_NOT) 

#define RSS1161x_OPMODE_100Tx ((ULONG)(       \
                                  RSS1161x_PORT_SELECT         \
                                | RSS1161x_HEARTBEAT_DISABLE   \
                                | RSS1161x_LINK_HYSTERESIS     \
                                | RSS1161x_DEFAULT_TXM_THRESHOLD \
                             ))
    

/* Initialize the hardware */

static int rss_hw_init( struct device *dev )
{
  DWORD connect, hwOpts;
  PHY_STATUS	status = PHY_STATUS_OK;
  struct rss_private *dv = (struct rss_private *)dev->priv;

  dv->CableModem = 1;		//JC debug, force to cable modem

  /* Reset NIC */
  outl(RSS_SW_RESET,dev->base_addr + RSS_PAR);
  rss_delay(50000);
  outl(0,dev->base_addr + RSS_PAR);
  rss_delay(80000);

  dv->rss_nar = RSS1161x_OPMODE_100Tx;

  connect = PHY_CONN_AUTONEG_ALL_MEDIA;
  //connect = PHY_CONN_100BASE_TX;
  hwOpts = 	 PHY_HWOPTS_NIC_RSS11623;
  hwOpts |= PHY_HWOPTS_GP0_SPI_DOUT;
  /* Enable MAC port management/hopping options */
  hwOpts |= (PHY_HWOPTS_MAC_PORT_MGMT | PHY_HWOPTS_PORT_HOPPING);
  hwOpts |= PHY_HWOPTS_MII_PORT_PRIORITY;
  hwOpts &= ~PHY_HWOPTS_ISOLATE;

  switch ( rss_tbl[dv->chip_id].rev_id )
  {
#if 0
	case RSS_REV3:
  		status = PhyInit(dev->base_addr, &dv->phy, PHY_ADDR_ANY,
			 connect, PHY_HWOPTS_NIC_RSS11610); 
		break;
	case RSS_REV4:
  		status = PhyInit(dev->base_addr, &dv->phy, PHY_ADDR_ANY,
			 connect, PHY_HWOPTS_NIC_RSS11611); 
		break;
	case RSS_REV5:
  		status = PhyInit(dev->base_addr, &dv->phy, PHY_ADDR_ANY,
			 connect, PHY_HWOPTS_NIC_RSS11617); 
		break;
#endif
	case RSS_REV8:
		if ( dv->CableModem )
		{
			printk("Cable Modem\n");
			//Set MAC to 100FDX MII mode
			dv->rss_nar &= ~(RSS1161x_INTERNAL_HLAN_SELECT | RSS1161x_NETWORK_SPEED_10);
			dv->rss_nar |= RSS1161x_FULL_DUPLEX_MODE;
			dv->rss_nar |= RSS1161x_PORT_SELECT;				//Force MII mode
		}
		else
		{
  			status = PhyInit(dev->base_addr, &dv->phy, PHY_ADDR_ANY,
  			    connect, hwOpts, (DWORD *)&dv->rss_nar);
		}
		break;
	default:
  		status = PhyInit(dev->base_addr, &dv->phy, PHY_ADDR_ANY,
  		    connect, hwOpts, (DWORD *)&dv->rss_nar);
			
		break;
  }

  if ( status != PHY_STATUS_OK )
  {
    printk ("PHY_STATUS = NOT O.K\n");
    return (FALSE);
  }

  outl(dv->rss_nar, dev->base_addr + RSS_NAR);
  return (TRUE);

}


static int rss_open(struct device *dev)
{
	struct rss_private *dv = (struct rss_private *)dev->priv;
	int ioaddr = dev->base_addr;

#ifdef _LINUX_DELAY_H
	udelay(2);
#else
	SLOW_DOWN_IO;
#endif
	/* Deassert reset.
	   486: Set 8 longword cache alignment, 8 longword burst.
	   586: Set 16 longword cache alignment, no burst limit.
	   Cache alignment bits 15:14	     Burst length 13:8
		0000	No alignment  0x00000000 unlimited		0800 8 longwords
		4000	8  longwords		0100 1 longword		1000 16 longwords
		8000	16 longwords		0200 2 longwords	2000 32 longwords
		C000	32  longwords		0400 4 longwords
	   Wait the specified 50 PCI cycles after a reset by initializing
	   Tx and Rx queues and the address filter list. */
#if defined(__alpha)
	/* ToDo: Alpha setting could be better. */
	outl(0x00200000 | 0xE000, ioaddr + RSS_PAR);
#else
#ifdef RW1161X_NIC 
	outl(0x00200000 | 0x4800, ioaddr+ RSS_PAR);	/* set cache alignment & DMA burst */
#else
#if defined(MODULE)
	/* When a module we don't have 'x86' to check. */
	outl(0x00200000 | 0x4800, ioaddr + RSS_PAR);

#else
	outl(0x00200000 | (x86 <= 4 ? 0x4800 : 0x8000), ioaddr + RSS_PAR);
	if ( x86 <= 4 )
	{
	  printk("This is a 386/486 PCI system, setting cache alignment to %x.\n",
			 0x00200000 | (x86 <= 4 ? 0x4800 : 0x8000));
	}		
#endif
#endif
#endif

#ifdef SA_SHIRQ
	if ( request_irq(dev->irq, &rss_interrupt, SA_SHIRQ, rss_tbl[dv->chip_id].chip_name, dev) )
    {
		return -EAGAIN;
	}
#else
	if ( irq2dev_map[dev->irq] != NULL ||
	    (irq2dev_map[dev->irq] = dev) == NULL ||
	     dev->irq == 0 ||
	     request_irq(dev->irq, &rss_interrupt, 0, rss_tbl[dv->chip_id].chip_name) )
    {
		return -EAGAIN;
	}
#endif

	if ( rss_debug > 1 )
	{
		printk("%s: rss_open() irq %d.\n", dev->name, dev->irq);
    }	

	MOD_INC_USE_COUNT;

    rss_setup_mem(dev);

	return 0;
}

static void rss_setup_mem( struct device *dev )
{
	struct rss_private *dv = (struct rss_private *)dev->priv;
	int ioaddr = dev->base_addr;

	// initialize the circular ring buffers
	rss_init_ring(dev);
	dv->tx_timeout = 0;

	/* This is set_rx_mode(), but without starting the transmitter. */
	/* Fill the whole address filter table with our physical address. */
	{
		u16 *eaddrs = (u16 *)dev->dev_addr;
		int *setup_frm = dv->setup_frame, i;

		/* You must add the broadcast address when doing perfect filtering! */
		*setup_frm++ = 0xffff;
		*setup_frm++ = 0xffff;
		*setup_frm++ = 0xffff;
		/* Fill the rest of the accept table with our physical address. */
		for ( i = 1; i < 16; i++ )
		{

			*setup_frm++ = eaddrs[0];
			*setup_frm++ = eaddrs[1];
			*setup_frm++ = eaddrs[2];
		}
		
		/* Put the setup frame on the Tx list. */
		dv->tx_ring[0].length = 0x08000000 | 192;
		dv->tx_ring[0].buffer1 = virt_to_bus(dv->setup_frame);
		dv->tx_ring[0].status = 0x80000000;

		dv->cur_tx++;
	}

	outl(virt_to_bus(dv->rx_ring), ioaddr + RSS_RDB);
	outl(virt_to_bus(dv->tx_ring), ioaddr + RSS_TDB);

	/* Start the chip's Tx to process setup frame. */
	dv->rss_nar &= ~0x2002;
	outl(dv->rss_nar, ioaddr + RSS_NAR);
	dv->rss_nar |= 0x2000;
	outl(dv->rss_nar, ioaddr + RSS_NAR);

	#if LINUX_VERSION_CODE < 0x020400
		dev->start = 1;
		dev->tbusy = 0;
		dev->interrupt = 0;
        #else
                netif_start_queue(dev);
	#endif
	
	/* Enable interrupts by setting the interrupt mask. */
	outl(0x0001c84f, ioaddr + RSS_IER1);
	dv->rss_nar |= 0x2002;
	outl(dv->rss_nar, ioaddr + RSS_NAR);
	outl(0, ioaddr + RSS_RDR);		/* Rx poll demand */

	if ( rss_debug > 2 )
	{
		printk("%s: Done rss_open(), RSS_PAR %8.8x, RSS_SR1 %8.8x \n",
			   dev->name, inl(ioaddr + RSS_PAR), inl(ioaddr + RSS_SR1));
			 
	}
	/* Set the timer to switch to check for link beat and perhaps switch
	   to an alternate media type. */
	init_timer(&dv->timer);
	dv->timer.expires = RUN_AT((24*HZ)/10);			/* 2.4 sec. */
	dv->timer.data = (unsigned long)dev;
	dv->timer.function = &rss_timer;			/* timer handler */
	add_timer(&dv->timer);
}

static void FreeAllTxBuf( struct device *dev )
{
	struct rss_private *dv = (struct rss_private *)dev->priv;
	int dirty_tx;

	for (dirty_tx = dv->dirty_tx; dirty_tx < dv->cur_tx; dirty_tx++)
	{
		int entry = dirty_tx % TX_RING_SIZE;

		dv->stats.tx_bytes += dv->tx_ring[entry].length & 0x7ff;
		dv->stats.tx_packets++;
		/* Free the original skb. */
		dev_free_skb(dv->tx_skbuff[entry]);
		dv->tx_skbuff[entry] = 0;
	}
	
	#if LINUX_VERSION_CODE < 0x020400
	if ( dv->tx_full && dev->tbusy )
	{
		/* The ring is no longer full, clear tbusy. */
		dv->tx_full = 0;
		dev->tbusy = 0;
		mark_bh(NET_BH);
	}
        #else
        if ( dv->tx_full)
        {
                /*The ring is no longer full, clear tbusy. */
                dv->tx_full = 0;
                netif_wake_queue(dev);
        }
	#endif
	
	dv->dirty_tx = dirty_tx;
}

/* Set up the transceiver control registers for the selected media type. */
static void rss_timer( unsigned long data )
{
	struct device *dev = (struct device *)data;
	struct rss_private *dv = (struct rss_private *)dev->priv;
	int ioaddr = dev->base_addr;
	int next_tick = 0;
	int rss_sr1,i;

	rss_sr1 = inl(ioaddr + RSS_SR1);

	if ( dv->tx_timeout && dv->tx_full )
	{
		if ( rss_debug > 0 )
		{
			printk("tx_timeout, %x\n", rss_sr1);		//JC debug
        }
        		
		if ( (rss_sr1 & 0x700000) == TX_HANG || (rss_sr1 & 0x700000) == TX_DMAING )
		{
			if ( rss_debug > 0 )
			{
				printk("Tx hang, %x\n", rss_sr1);		//JC debug
            }				
			FreeAllTxBuf(dev);
			rss_reset(dev);
			return;
		}
	}
	
	if ( (rss_sr1 & 0xE0000) == RX_STATE_4 )
	{
		i = dv->cur_rx;
		if ( dv->rx_ring[i].status & RDES_OWN )
		{
			if (rss_debug > 0)
			{
				printk("Rcv hang, %x\n", rss_sr1);		//JC debug
            }				
			FreeAllTxBuf(dev);
			rss_reset(dev);
			return;
		}
	}
	
	next_tick = (24*HZ)/10;
	if ( next_tick )
	{
		dv->timer.expires = RUN_AT(next_tick);
		add_timer(&dv->timer);
	}
    rss_phy_check(dev);
}

static void rss_tx_timeout( struct device *dev )
{
  struct rss_private *dv = (struct rss_private *)dev->priv;
  int ioaddr = dev->base_addr;

  int status = inl(ioaddr + RSS_SR1);

  dv->tx_timeout = 1;

  /* Stop the transmit process. */
  dv->rss_nar &= ~0x2000;
  outl(dv->rss_nar, ioaddr + RSS_NAR);
  if ( rss_debug > 0 )
  {
	printk("%s: cn7112 transmit timed out, status %8.8x, SIA %8.8x %8.8x, resetting...\n",
		   dev->name, status,  inl(ioaddr + RSS_TDR),
		   inl(ioaddr + RSS_SR1));
  }		

  /* Restart the chip's Tx processes . */
  dv->rss_nar |= 0x2002;
  outl(dv->rss_nar, ioaddr + RSS_NAR);
  /* Trigger an immediate transmit demand. */
  outl(0, ioaddr + RSS_TDR);
  dev->trans_start = jiffies;
  dv->stats.tx_errors++;
}

static void rss_reset( struct device *dev )
{
  DWORD phy_data;

  if ( rss_debug > 0 )
  {
	  printk ("RSS_reset\n");
  }
	
  rss_hw_init(dev);
  rss_setup_mem(dev);

  phy_data = rss_phy_check(dev);
  if ( !(phy_data & PHY_LINK_UP) )
  {
	  if (rss_debug > 0)
	  {
		printk ("rss: cable disconnect error.\n");
	  }
  }
}


/* Initialize the Rx and Tx rings, along with various 'dev' bits. */
static void rss_init_ring(struct device *dev)
{
	struct rss_private *dv = (struct rss_private *)dev->priv;
	int i;

	dv->tx_full = 0;
	dv->cur_rx = dv->cur_tx = 0;
	dv->dirty_rx = dv->dirty_tx = 0;

	for ( i = 0; i < RX_RING_SIZE; i++ )
	{
		dv->rx_ring[i].status = 0x80000000;	/* Owned by RWNIC chip */
		dv->rx_ring[i].length = PKT_BUF_SZ;
		{
			/* Note the receive buffer must be longword aligned.
			   dev_alloc_skb() provides 16 byte alignment.  But do *not*
			   use skb_reserve() to align the IP header! */
			struct sk_buff *skb;
			skb = DEV_ALLOC_SKB(PKT_BUF_SZ);
			dv->rx_skbuff[i] = skb;
			if ( skb == NULL )
			{
				break;			/* Bad news!  */
			}
			skb->dev = dev;			/* Mark as being used by this device. */
#if LINUX_VERSION_CODE > 0x10300
			dv->rx_ring[i].buffer1 = virt_to_bus(skb->tail);
#else
			dv->rx_ring[i].buffer1 = virt_to_bus(skb->data);
#endif
		}
//		dv->rx_ring[i].buffer2 = virt_to_bus(&dv->rx_ring[i+1]);
		dv->rx_ring[i].buffer2 = 0;
	}
	/* Mark the last entry as wrapping the ring. */
	dv->rx_ring[i-1].length = PKT_BUF_SZ | 0x02000000;
//	dv->rx_ring[i-1].buffer2 = virt_to_bus(&dv->rx_ring[0]);

	/* The Tx buffer descriptor is filled in as needed, but we
	   do need to clear the ownership bit. */
	for ( i = 0; i < TX_RING_SIZE; i++ )
	{
		dv->tx_skbuff[i] = 0;
		dv->tx_ring[i].status = 0x00000000;
		//dv->tx_ring[i].buffer2 = virt_to_bus(&dv->tx_ring[i+1]);
		dv->tx_ring[i].buffer2 = 0;
	}
	//dv->tx_ring[i-1].buffer2 = virt_to_bus(&dv->tx_ring[0]);
    /* mark the end of the chain */
    dv->tx_ring[TX_RING_SIZE-1].length |= 0x02000000;
}


static int rss_start_xmit( struct sk_buff *skb, struct device *dev )
{
	struct rss_private *dv = (struct rss_private *)dev->priv;
	int entry;
	u32 flag;

        if (rss_debug > 3)
           printk("rss_start_xmit\n");

	/* Block a timer-based transmit from overlapping.  This could better be
	   done with atomic_swap(1, dev->tbusy), but set_bit() works as well. */
        /* 2.4 guarantees that there won't be multiple threads in this
           method at once */	
	#if LINUX_VERSION_CODE < 0x020400
	if ( test_and_set_bit(0, (void*)&dev->tbusy) != 0 )
	{
		if ( jiffies - dev->trans_start < TX_TIMEOUT )
		{
			return 1;
		}

		rss_tx_timeout(dev);
		return 1;
	}
	#endif

	/* Caution: the write order is important here, set the base address
	   with the "ownership" bits last. */

	/* Calculate the next Tx descriptor entry. */
	entry = dv->cur_tx % TX_RING_SIZE;

	dv->tx_skbuff[entry] = skb;
	dv->tx_ring[entry].buffer1 = virt_to_bus(skb->data);

	if ( dv->cur_tx - dv->dirty_tx < TX_RING_SIZE / 2 )
	{/* Typical path */
	  flag = 0x60000000; /* No interrupt */
	}
	else if ( dv->cur_tx - dv->dirty_tx == TX_RING_SIZE / 2 )
	{
	  flag = 0xe0000000; /* Tx-done intr. */
	}
	else if ( dv->cur_tx - dv->dirty_tx < TX_RING_SIZE - 2 )
	{
	  flag = 0x60000000; /* No Tx-done intr. */
	}
	else
	{
	  /* Leave room for set_rx_mode() to fill entries. */
	  flag = 0xe0000000; /* Tx-done intr. */
	  dv->tx_full = 1;
	}
	
	if ( entry == TX_RING_SIZE - 1 )
	{
		flag |= 0xe2000000;
	}

	dv->tx_ring[entry].length = skb->len | flag;
	dv->tx_ring[entry].status = 0x80000000;	/* Pass ownership to the chip. */
	dv->cur_tx++;
	

	#if LINUX_VERSION_CODE < 0x020400
	if ( !dv->tx_full )
	{
		clear_bit(0, (void*)&dev->tbusy);
	}
        #else
        if ( dv->tx_full )
        {
                netif_stop_queue(dev);
        }
	#endif	
	
	/* Trigger an immediate transmit demand. */
	outl(0, dev->base_addr + RSS_TDR);

	// timestamp the packet
	dev->trans_start = jiffies;

#if 0
	//JC debug
	printk("TxDesc[-1] = %x %x %x %x\n",
	       dv->tx_ring[entry - 1].status,
	       dv->tx_ring[entry - 1].length,
	       dv->tx_ring[entry - 1].buffer1,
	       dv->tx_ring[entry - 1].buffer2);
	printk("TxDesc @%x = %x %x %x %x\n", &dv->tx_ring[entry],
	       dv->tx_ring[entry].status,
	       dv->tx_ring[entry].length,
	       dv->tx_ring[entry].buffer1,
	       dv->tx_ring[entry].buffer2);
	printk("TxBuf @%x (%x phys) = %x %x %x %x %x %x",
	       skb->data, virt_to_bus(skb->data),
		   skb->data[0],
		   skb->data[1],
		   skb->data[2],
		   skb->data[3],
		   skb->data[4],
		   skb->data[5]);
	printk("%x %x %x %x %x %x\n",
		   skb->data[6],
		   skb->data[7],
		   skb->data[8],
		   skb->data[9],
		   skb->data[10],
		   skb->data[11]);
#endif

	return 0;
}


/* The interrupt handler does all of the Rx thread work and cleans up
   after the Tx thread. */
static void rss_interrupt IRQ( int irq, void *dev_instance, struct pt_regs *regs )
{
#ifdef SA_SHIRQ		/* Use the now-standard shared IRQ implementation. */
	struct device *dev = (struct device *)dev_instance;
#else
	struct device *dev = (struct device *)(irq2dev_map[irq]);
#endif

	struct rss_private *lp;
	int rss_sr1, ioaddr, boguscnt = 10;

	if ( dev == NULL )
	{
		printk ("rss_interrupt(): irq %d for unknown device.\n", irq);
		return;
	}
	
	ioaddr = dev->base_addr;
	lp = (struct rss_private *)dev->priv;

        /* 2.4 guarantees only one thread in here at a time */
	#if LINUX_VERSION_CODE < 0x020400
	if ( dev->interrupt )
	{
		printk("%s: Re-entering the interrupt handler.\n", dev->name);
		dev->interrupt = 1;
	}		
	#endif

	do
	{
		// read status register
		rss_sr1 = inl(ioaddr + RSS_SR1);

		if ( rss_debug > 3 )
		{
			printk("rss_sr1=%x\n", rss_sr1);
		}

		/* Acknowledge all of the current interrupt sources ASAP. */
		outl(rss_sr1 & 0x0001ffff, ioaddr + RSS_SR1);

		if ( rss_debug > 4 )
		{
			printk("%s: interrupt  rss_sr1=%#8.8x new rss_sr1=%#8.8x.\n",
				   dev->name, rss_sr1, inl(dev->base_addr + RSS_SR1));
		}

		if ( (rss_sr1 & 0x00018000) == 0 )
		{
			break;
		}

		if ( rss_sr1 & 0x0040 )			/* Rx interrupt */
		{
			rss_rx(dev);
		}
#ifdef RW1161X_NIC
		if ( rss_sr1 & 0x0027 )
		{
#else
			if ( rss_sr1 & 0x0007 )
			{	/* Tx-done interrupt */
#endif
				int dirty_tx;

	                        if (rss_debug > 4)
              	                   printk(" In rss_interrupt(), Tx-done interrupt\n");

				for ( dirty_tx = lp->dirty_tx; dirty_tx < lp->cur_tx; dirty_tx++ )
				{
					int entry = dirty_tx % TX_RING_SIZE;
					int status = lp->tx_ring[entry].status;

					if ( status < 0 )
					{
						break;			/* It still hasn't been Txed */
					}
					/* Check for Rx filter setup frames. */
					if ( lp->tx_skbuff[entry] == NULL )
					{
					  continue;
    			    }

/* the rs7112 falsely reports loss of carrier (0x800) and
   no carrier (0x400) on every packet, and link error
   (0x4) on every other packet -- filter those errors out */
#define TX_LOSS_OF_CARRIER 0x800
#define TX_NO_CARRIER 0x400
#define TX_LINK_ERROR 0x4
#define TX_BOGUS_ERRORS (TX_LOSS_OF_CARRIER | TX_NO_CARRIER | TX_LINK_ERROR)
#define TX_ERROR_MASK 0x7fff
#define TX_VALID_ERRORS (TX_ERROR_MASK & ~TX_BOGUS_ERRORS)

					if (( status & 0x8000 ) & ( status & TX_VALID_ERRORS ))
					{
						/* There was an major error, log it. */
#ifndef final_version
						if ( rss_debug > 1 )
						{
							printk("%s: Transmit error, Tx status %8.8x.\n",
								   dev->name, status);
					    }
#endif
					    lp->stats.tx_errors++;
/*						if ( status & 0x4104 ) lp->stats.tx_aborted_errors++;
						if ( status & 0x0C00 ) lp->stats.tx_carrier_errors++;
*/                                              if ( status & 0x4100 ) lp->stats.tx_aborted_errors++;
						if ( status & 0x0200 ) lp->stats.tx_window_errors++;
						if ( status & 0x0002 ) lp->stats.tx_fifo_errors++;
						if ( (status & 0x0080) && lp->full_duplex == 0 ) lp->stats.tx_heartbeat_errors++;
#ifdef ETHER_STATS
						if ( status & 0x0100 ) lp->stats.collisions16++;
#endif
					}
					else
					{
#ifdef ETHER_STATS
						if (status & 0x0001) lp->stats.tx_deferred++;
#endif
#if LINUX_VERSION_CODE > 0x20127
						lp->stats.tx_bytes += lp->tx_ring[entry].length & 0x7ff;
#endif
						lp->stats.collisions += (status >> 3) & 15;
						lp->stats.tx_packets++;
					}
					
					/* Free the original skb. */
					lp->tx_timeout = 0;
					
					#if LINUX_VERSION_CODE >= 0x020400
					dev_kfree_skb_irq(lp->tx_skbuff[entry]);
					#else
					dev_free_skb(lp->tx_skbuff[entry]);
					#endif
					lp->tx_skbuff[entry] = 0;
				}

#ifndef final_version
				if ( lp->cur_tx - dirty_tx > TX_RING_SIZE )
				{
					printk("%s: Out-of-sync dirty pointer, %d vs. %d, full=%d.\n",
						   dev->name, dirty_tx, lp->cur_tx, lp->tx_full);
					dirty_tx += TX_RING_SIZE;
				}
#endif
			#if LINUX_VERSION_CODE < 0x020400
			if ( lp->tx_full && dev->tbusy && dirty_tx > lp->cur_tx - TX_RING_SIZE + 2 )
			{
				/* The ring is no longer full, clear tbusy. */
				lp->tx_full = 0;
				dev->tbusy = 0;
				mark_bh(NET_BH);
			}
                        #else
			if ( lp->tx_full && dirty_tx > lp->cur_tx - TX_RING_SIZE + 2 )
                        {
				/* The ring is no longer full, clear tbusy. */
				lp->tx_full = 0;
                                /* may be called redundantly? */
                                netif_wake_queue(dev);
                        }
			#endif

			lp->dirty_tx = dirty_tx;
		}

		/* Log errors. */
		if ( rss_sr1 & 0x8000 )
		{	
			/* Abnormal error summary bit. */
                        if ( rss_debug > 4 )
           	           printk(" In rss_interrupt(), abnormal error summary bit was set.\n");

			if ( rss_sr1 & 0x0008 )
				lp->stats.tx_errors++; /* Tx babble. */
			
			if ( rss_sr1 & 0x0020 )
			{ /* Tx FIFO underflow. */
			  lp->rss_nar |= 0x00200000;  /* Reconfigure to store-n-forward. */
			  /* Restart the transmit process. */
              lp->rss_nar &= ~0x2000; 
			  outl(lp->rss_nar, ioaddr + RSS_NAR);
              lp->rss_nar |= 0x2002; 
			  outl(lp->rss_nar, ioaddr + RSS_NAR);
			}
			
			if ( rss_sr1 & 0x0100 )
			{		/* Missed a Rx frame. */
				lp->stats.rx_errors++;
				lp->stats.rx_missed_errors += inl(ioaddr + RSS_LPC) & 0xffff;
			}
			
			if ( rss_sr1 & 0x0800 )
			{
				printk("%s: Something Wicked happened! %8.8x.\n",
					   dev->name, rss_sr1);
				/* Hmmmmm, it's not clear what to do here. */
			}
			/* Clear all error sources, included undocumented ones! */
			outl(0x000f7ba, ioaddr + RSS_SR1);
		}
		
		if ( --boguscnt < 0 )
		{
			if ( rss_debug > 0 )
			{
				printk("%s: Too much work at interrupt, rss_sr1=0x%8.8x.\n",
					   dev->name, rss_sr1);
 		    }
			/* Clear all interrupt sources. */
			outl(0x0001ffff, ioaddr + RSS_SR1);
			break;
		}
		
	} while( 1 );

	if ( rss_debug > 3 )
	{
		printk("%s: exiting interrupt, rss_sr1=%#4.4x.\n",
			   dev->name, inl(ioaddr + RSS_SR1));
	}

	#if LINUX_VERSION_CODE < 0x020400
	dev->interrupt = 0;
	#endif
	
	return;
}


static int rss_rx( struct device *dev )
{
	struct rss_private *lp = (struct rss_private *)dev->priv;
	int entry = lp->cur_rx % RX_RING_SIZE;

	if ( rss_debug > 4 )
	{
		printk(" In rss_rx(), entry %d %8.8x.\n", entry, lp->rx_ring[entry].status);
	}
	
	/* If we own the next entry, it's a new packet. Send it up. */
	while ( lp->rx_ring[entry].status >= 0 )
	{
		int status = lp->rx_ring[entry].status;

		if ( rss_debug > 4 )
		{
			printk("  rss_rx() status was %8.8x.\n", status);
		}
			
		if ( (status & 0x0300) != 0x0300 )
		{
			if ( (status & 0xffff) != 0x7fff )
			{ /* Ingore earlier buffers. */
			  printk("%s: Oversized Ethernet frame spanned multiple buffers,"
					 " status %8.8x!\n", dev->name, status);
			  lp->stats.rx_length_errors++;
			}
		}
		else if ( status & 0x8000 )
		{
			/* There was a fatal error. */
			lp->stats.rx_errors++; /* end of a packet.*/
			if ( status & 0x0890 ) lp->stats.rx_length_errors++;
			if ( status & 0x0004 ) lp->stats.rx_frame_errors++;
			if ( status & 0x0002 ) lp->stats.rx_crc_errors++;
			if ( status & 0x0001 ) lp->stats.rx_fifo_errors++;
		}
		else
		{
			/* Malloc up new buffer, compatible with net-2e. */
			/* Omit the four octet CRC from the length. */
			short pkt_len = (lp->rx_ring[entry].status >> 16) - 4;
			struct sk_buff *skb;
			int rx_in_place = 0;

			/* Check if the packet is long enough to just accept without
			   copying to a properly sized skbuff. */
			if ( pkt_len > SKBUFF_RX_COPYBREAK )
			{
				struct sk_buff *newskb;
				char *temp;

				/* Pass up the skb already on the Rx ring. */
				skb = lp->rx_skbuff[entry];
				temp = skb_put(skb, pkt_len);
				if ( bus_to_virt(lp->rx_ring[entry].buffer1 ) != temp)
				{
					printk("%s: Warning -- the skbuff addresses do not match"
						   " in rss_rx: %p vs. %p / %p.\n", dev->name,
						   bus_to_virt(lp->rx_ring[entry].buffer1),
						   skb->head, temp);
			   }
			
				/* Get a fresh skbuff to replace the filled one. */
				newskb = DEV_ALLOC_SKB(PKT_BUF_SZ);
				if ( newskb )
				{
					rx_in_place = 1;
					lp->rx_skbuff[entry] = newskb;
					newskb->dev = dev;
					/* Longword alignment required: do not skb_reserve(2)! */
					lp->rx_ring[entry].buffer1 = virt_to_bus(newskb->tail);
				}
				else			/* No memory, drop the packet. */
				{
				  skb = 0;
			    }
			}
			else
			{
				skb = DEV_ALLOC_SKB(pkt_len + 2);
			}
			
			if ( skb == NULL )
			{
				int i;
				printk("%s: Memory squeeze, deferring packet.\n", dev->name);
				/* Check that at least two ring entries are free.
				   If not, free one and mark stats->rx_dropped++. */

				for ( i = 0; i < RX_RING_SIZE; i++ )
				{
					if ( lp->rx_ring[(entry+i) % RX_RING_SIZE].status < 0 )
					{
						break;
					}
				}					
	
				if ( i > RX_RING_SIZE - 2 )
				{
					lp->stats.rx_dropped++;
					lp->rx_ring[entry].status = 0x80000000;
					lp->cur_rx++;
				}
				break;
			}
			
			skb->dev = dev;
			
			if ( !rx_in_place )
			{
				skb_reserve(skb, 2);	/* 16 byte align the data fields */
				memcpy(skb_put(skb, pkt_len), bus_to_virt(lp->rx_ring[entry].buffer1), pkt_len);
			}
#if LINUX_VERSION_CODE > 0x10300
			skb->protocol = eth_type_trans(skb, dev);
#else
			skb->len = pkt_len;
#endif
			netif_rx(skb);
			lp->stats.rx_packets++;
#if LINUX_VERSION_CODE > 0x20127
			lp->stats.rx_bytes += pkt_len;
#endif
		}

		lp->rx_ring[entry].status = 0x80000000;
		entry = (++lp->cur_rx) % RX_RING_SIZE;
	}

	return 0;
}


static int rss_close( struct device *dev )
{
	int ioaddr = dev->base_addr;
	struct rss_private *dv = (struct rss_private *)dev->priv;
	int i;

	#if LINUX_VERSION_CODE < 0x020400
		dev->start = 0;
        #else
                netif_stop_queue(dev);
	#endif
		if (rss_debug > 1)
		{
			printk("%s: Shutting down ethercard, status was %2.2x.\n",
				   dev->name, inl(ioaddr + RSS_SR1));
	    }
	
	/* Disable interrupts by clearing the interrupt mask. */
	outl(0x00000000, ioaddr + RSS_IER1);
	/* Stop the chip's Tx and Rx processes. */
	dv->rss_nar &= ~0x2002;
	outl(dv->rss_nar, ioaddr + RSS_NAR);

	dv->stats.rx_missed_errors += inl(ioaddr + RSS_LPC) & 0xffff;

	del_timer(&dv->timer);

#ifdef SA_SHIRQ
	free_irq(dev->irq, dev);
#else
	free_irq(dev->irq);
	irq2dev_map[dev->irq] = 0;
#endif

	/* Free all the skbuffs in the Rx queue. */
	for ( i = 0; i < RX_RING_SIZE; i++ )
	{
		struct sk_buff *skb = dv->rx_skbuff[i];
		dv->rx_skbuff[i] = 0;
		dv->rx_ring[i].status = 0;		/* Not owned by RWNIC chip. */
		dv->rx_ring[i].length = 0;
		dv->rx_ring[i].buffer1 = 0xBADF00D0; /* An invalid address. */
		if ( skb )
		{
#if LINUX_VERSION_CODE < 0x20100
			skb->free = 1;
#endif
			dev_free_skb(skb);
		}
	}
	
	for ( i = 0; i < TX_RING_SIZE; i++ )
	{
		if ( dv->tx_skbuff[i] )
		{
			dev_free_skb(dv->tx_skbuff[i]);
		}
			
		dv->tx_skbuff[i] = 0;
	}

	MOD_DEC_USE_COUNT;

	return 0;
}


static struct net_device_stats *rss_get_stats( struct device *dev )
{
	struct rss_private *dv = (struct rss_private *)dev->priv;
	int ioaddr = dev->base_addr;

#if LINUX_VERSION_CODE >= 0x020400
	if( netif_running(dev) )
#else
	if ( dev->start )
#endif
	{
		unsigned long lost_packets = inl(ioaddr + RSS_LPC) & 0xffff	;
		dv->stats.rx_missed_errors += lost_packets;
	}

	return &dv->stats;
}

/* Set or clear the multicast filter for this adaptor.
   Note that we only use exclusion around actually queueing the
   new frame, not around filling dv->setup_frame.  This is non-deterministic
   when re-entered but still correct. */

/* The little-endian AUTODIN32 ethernet CRC calculation.
   N.B. Do not use for bulk data, use a table-based routine instead.
   This is common code and should be moved to net/core/crc.c */

static unsigned const ethernet_polynomial_le = 0xedb88320U;

static inline unsigned ether_crc_le(int length, unsigned char *data)
{
	unsigned int crc = 0xffffffff;	/* Initial value. */
	while( --length >= 0 )
	{
		unsigned char current_octet = *data++;
		int bit;
		
		for ( bit = 8; --bit >= 0; current_octet >>= 1 )
		{
			if ( (crc ^ current_octet) & 1 )
			{
				crc >>= 1;
				crc ^= ethernet_polynomial_le;
			}
			else
			{
				crc >>= 1;
			}
		}
	}
	
	return crc;
}



static void
#ifdef NEW_MULTICAST
set_multicast_list( struct device *dev )
#else
static void set_multicast_list( struct device *dev, int num_addrs, void *addrs );
#endif
{
	int ioaddr = dev->base_addr;
	struct rss_private *dv = (struct rss_private *)dev->priv;

	dv->rss_nar = inl(ioaddr + RSS_NAR) & ~0x00D5;
	if ( dev->flags & IFF_PROMISC )
	{			/* Set promiscuous. */
		/* Unconditionally log net taps. */
		printk("%s: Promiscuous mode enabled.\n", dev->name);
		dv->rss_nar |= 0xC0;
		outl(dv->rss_nar, ioaddr + RSS_NAR);
	}
	else if ( (dev->mc_count > 1000) || (dev->flags & IFF_ALLMULTI) )
	{
		/* Too many to filter perfectly -- accept all multicasts. */
		dv->rss_nar |= 0x80;
		outl(dv->rss_nar, ioaddr + RSS_NAR);
	}
	else
	{
		u32 *setup_frm = dv->setup_frame;
		struct dev_mc_list *mclist;
		u16 *eaddrs;
		u32 tx_flags;
		int i;

		dv->rss_nar &= ~0xC0;
		if ( dev->mc_count > 14 )
		{ /* Must use a multicast hash table. */
		  u16 hash_table[32];
		  memset(hash_table, 0, sizeof(hash_table));
		
		  for ( i = 0, mclist = dev->mc_list; mclist && i < dev->mc_count; i++, mclist = mclist->next )
		  {
			set_bit(ether_crc_le(ETH_ALEN, mclist->dmi_addr) & 0x1ff, hash_table);
		  }
		
		  /* Copy the hash table to the setup frame.
			 NOTE that only the LOW SHORTWORD of setup_frame[] is valid!
			 This code may require tweaking for non-x86 architectures!  */
		  for ( i = 0; i < 32; i++ )
		  {
			*setup_frm++ = hash_table[i];
		  }
		  setup_frm += 7;
		  tx_flags = 0x08400000 | 192;
		  /* Too clever: i > 15 for fall-though. */
		
		}
		else
		{
		  /* We have <= 15 addresses so we can use the wonderful
			 16 address perfect filtering of the RWNIC. */
		  for ( i = 0, mclist = dev->mc_list; i < dev->mc_count; i++, mclist = mclist->next )
	      {
			/* Note that only the low shortword of setup_frame[] is valid!
			   This code may require tweaking for non-x86 architectures! */
			eaddrs = (u16 *)mclist->dmi_addr;
			*setup_frm++ = *eaddrs++;
			*setup_frm++ = *eaddrs++;
			*setup_frm++ = *eaddrs++;
		  }
		
		  /* Fill the rest of the table with our physical address.
			 Once again, only the low shortword or setup_frame[] is valid! */
		  *setup_frm++ = 0xffff;
		  *setup_frm++ = 0xffff;
		  *setup_frm++ = 0xffff;
		  tx_flags = 0x08000000 | 192;
		}
		
		eaddrs = (u16 *)dev->dev_addr;
		
		do
		{
		  *setup_frm++ = eaddrs[0];
		  *setup_frm++ = eaddrs[1];
		  *setup_frm++ = eaddrs[2];
		} while ( ++i < 15 );
		
		/* Now add this frame to the Tx list. */
		if ( dv->cur_tx - dv->dirty_tx > TX_RING_SIZE - 2 )
		{
			/* Same setup recently queued, we need not add it. */
		}
		else
		{
			unsigned long flags;
			unsigned int entry;
			
			save_flags(flags); cli();
			entry = dv->cur_tx++ % TX_RING_SIZE;

			if ( entry != 0 )
			{
			  /* Avoid a chip errata by prefixing a dummy entry. */
			  dv->tx_skbuff[entry] = 0;
			  dv->tx_ring[entry].length = (entry == TX_RING_SIZE-1) ? 0x02000000 : 0;
			  dv->tx_ring[entry].buffer1 = 0;
			  dv->tx_ring[entry].status = DESC_OWNED_BY_MAC;
			  entry = dv->cur_tx++ % TX_RING_SIZE;
			}

			dv->tx_skbuff[entry] = 0;
			/* Put the setup frame on the Tx list. */
			if ( entry == TX_RING_SIZE - 1 )
			{
			  tx_flags |= 0x02000000;		/* Wrap ring. */
			}
			
			dv->tx_ring[entry].length = tx_flags;
			dv->tx_ring[entry].buffer1 = virt_to_bus(dv->setup_frame);
			dv->tx_ring[entry].status = DESC_OWNED_BY_MAC;
			if ( dv->cur_tx - dv->dirty_tx >= TX_RING_SIZE - 2 )
			{
				dv->tx_full = 1;
			}
			restore_flags(flags);
			/* Trigger an immediate transmit demand. */
			outl(0, ioaddr + RSS_TDR);
		}
		outl(dv->rss_nar | 0x0000, ioaddr + RSS_NAR);
	}
	
}



/********************************************************
 * Check Phy and set Network Access Register(RSS_NAR)	*
 ********************************************************/
static int rss_phy_check( struct device *dev )
{ 
  struct rss_private *dv = (struct rss_private *)dev->priv;
  unsigned long nar = 0;
  DWORD phy_data;

  if ( dv->CableModem )
  {
	  phy_data = 0;
	  return (phy_data);
  }

  phy_data = PhyCheck(&dv->phy);
  if (phy_data & PHY_LINK_NEW_MEDIA)
  {
      nar = inl(dev->base_addr + RSS_NAR);
#if 0		//JC debug
      nar &= ~(NAR_PCS | NAR_SCR | NAR_PROM | NAR_MB0);
      nar |= NAR_PS | NAR_HBD | NAR_MB1 | NAR_TSF;
      if (phy_data & PHY_LINK_FDX)
      {
         nar |= NAR_FD;
	 	 dv->full_duplex = TRUE;
      }
      else
      {
		 dv->full_duplex = FALSE;
	  }

      nar |= ((phy_data & PHY_LINK_100) ? NAR_TTM_100 : NAR_TTM_10);
      outl(nar,dev->base_addr + RSS_NAR);
#endif 
      dv->rss_nar = nar;

  }
  return (phy_data);
}

#ifdef MODULE

/* An additional parameter that may be passed in... */
static int debug = -1;

int init_module( void )
{
	int cards_found;

	if ( debug >= 0 )
	{
		rss_debug = debug;
	}
	
	if ( rss_debug )
	{
		printk(version);
	}
	
	root_rss_dev = NULL;
	cards_found = rss_probe(0);

	return cards_found ? 0 : -ENODEV;
}

void cleanup_module( void )
{
	struct device *next_dev;

	/* No need to check MOD_IN_USE, as sys_delete_module() checks. */
	while ( root_rss_dev )
	{
		next_dev = ((struct rss_private *)root_rss_dev->priv)->next_module;
		unregister_netdev(root_rss_dev);
		release_region(root_rss_dev->base_addr, RSS_TOTAL_SIZE);
		kfree(root_rss_dev);
		root_rss_dev = next_dev;
	}
}

#endif  /* MODULE */

/*
 * Local variables:
 *  SMP-compile-command: "gcc -D__SMP__ -DMODULE -D__KERNEL__ -I/usr/src/linux/net/inet -Wall -Wstrict-prototypes -O6 -c rss61x.c `[ -f /usr/include/linux/modversions.h ] && echo -DMODVERSIONS`"
 *  compile-command: "gcc -DMODULE -D__KERNEL__ -Wall -Wstrict-prototypes -O6 -c tulip.c `[ -f /usr/include/linux/modversions.h ] && echo -DMODVERSIONS`"
 *  cardbus-compile-command: "gcc -DCARDBUS -DMODULE -D__KERNEL__ -I/usr/src/linux/net/inet -Wall -Wstrict-prototypes -O6 -c rss61x.c -o tulip_cb.o -I/usr/src/pcmcia-cs-3.0.9/include/"
 *  c-indent-level: 4
 *  c-basic-offset: 4
 *  tab-width: 4
 * End:
 */

