/*************************************************************************************************
 *                         PVCS Version Control Information                                      *
 *                                                                                               *
 * $Header:   P:\d878\HomeLAN\Archives\NdisDriver\SPI.H_v   1.2   May 12 1999 12:06:50   wangmx  $                            
 *                                                                                               *
 ************************************************************************************************/

/*
 *  Module Name: spi.h
 *  Author: Joe Bonaker
 *
 *  Copyright  1998 Conexant Systems, Inc.
 *  All rights reserved.
 *
 *  This software may be used and distributed according to the terms
 *  of the GNU Public License, incorporated herein
 */

/***********************************************************************
 *  Module Description: HomePNA Serial Peripheral Interface (SPI)
 *  primitives supported by underlying "mii.c" MII and adapter/OS
 *  services.
 *
 * $Log:   P:\d878\HomeLAN\Archives\NdisDriver\SPI.H_v  $
 * 
 *    Rev 1.2   May 12 1999 12:06:50   wangmx
 *  
 * Revision 1.2  1998/11/20 11:08:16  bonakeaj
 * - 11623 integrated HomeLAN PHY support
 * Revision 1.1  1998/10/09 23:22:53  bonakeaj
 * Initial revision
 **********************************************************************/

#ifndef _SPI_H_
#define _SPI_H_

#include "phy.h"
#include "mii.h"        /* OS-dependent mappings */

/* SpiGpioInit <bitnum> values, per LAN_SPR R15 (offset 0x78) */
#define LAN_GPIO0       0x00010000L     /* LAN/modem device GP2 */
#define LAN_GPIO1       0x00020000L     /* LAN/modem device GP3 */
#define LAN_GPIO2       0x00040000L     /* LAN/modem device GP4 */
#define LAN_GPIO3       0x00080000L     /* LAN/modem device GP5 */
#define LAN_GPIO_MASK   (LAN_GPIO0 | LAN_GPIO1 | LAN_GPIO2 | LAN_GPIO3)

/* Board-specific external HPNA PHY reset (default LAN_GPIO[1]) */
#ifndef HPNA_GPIO_RESET
#define HPNA_GPIO_RESET         PHY_HWOPTS_HWRESET_GP1  // LAN_GPIO1
#endif

/* Board-specific GPIO<->SPI chip select mapping */
#ifndef SPI_GPIO_DEFAULT
#define SPI_GPIO_DEFAULT        LAN_GPIO2	/* 11617 ref. design */
#endif

/* APIs used by PhyInit() */
void  SpiInit(PHY     *phy,
              PHY_PORT spi_port,
              DWORD    hwreset_options,
              DWORD    gpio_chipselect);
void  SpiWriteEnable(PHY *phy);
void  SpiWriteDisable(PHY *phy);

/* APIs available for general use, e.g., Diagnostic */
void  SpiWriteByte(PHY   *phy, BYTE byteAddr, BYTE  byteValue);
void  SpiWriteWord(PHY   *phy, BYTE byteAddr, WORD  wordValue);
void  SpiWriteDword(PHY  *phy, BYTE byteAddr, DWORD dwordValue);
BYTE  SpiReadByte(PHY    *phy, BYTE byteAddr);
WORD  SpiReadWord(PHY    *phy, BYTE byteAddr /* not wordAddr! */);
DWORD SpiReadDword(PHY   *phy, BYTE byteAddr /* not dwordAddr! */);

#define SpiInitR12(adapter, phy, hwreset, chipselect)   {       \
    phy->r12_gpio = TRUE;                                       \
    SpiInit((adapter), (phy), (hwreset), (chipselect));         \
}

#endif /* _SPI_H_ */
